/**
  **************************************************************************
  * @file     hid_iap_user.h
  * @version  v2.1.1
  * @date     2022-07-22
  * @brief    usb config header file
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

/* define to prevent recursive inclusion -------------------------------------*/
#ifndef __MSC_IAP_USER_H
#define __MSC_IAP_USER_H

#ifdef __cplusplus
extern "C" {
#endif

#include "msc_class.h"

/** @addtogroup UTILITIES_examples
  * @{
  */

/** @addtogroup USB_IAP_bootloader
  * @{
  */

/** @defgroup bootloader_definition
  * @{
  */

#define FLASH_APP_ADDRESS                0x08008000
#define IAP_UPGRADE_COMPLETE_FLAG        0x41544B38

#define FLASH_SIZE_REG()                 ((*(uint32_t *)0x1FFFF7E0) & 0xFFFF) /*Get Flash size*/
#define KB_TO_B(kb)                      ((kb) << 10)

#define SECTOR_SIZE_1K                   0x400
#define SECTOR_SIZE_2K                   0x800
#define SECTOR_SIZE_4K                   0x1000

#define FLASH_SECTOR_1K_ALLGNED          0x3FF
#define FLASH_SECTOR_2K_ALLGNED          0x7FF
#define FLASH_SECTOR_4K_ALLGNED          0xFFF

/**
  * @}
  */

/** @defgroup bootloader_exported_functions
  * @{
  */
typedef struct
{

  uint8_t iap_fifo[4096];

  uint32_t fifo_length;

  uint32_t app_address;
  uint32_t iap_address;
  uint32_t flag_address;

  uint32_t flash_start_address;
  uint32_t flash_end_address;

  uint32_t sector_size;
  uint32_t sector_msk;
  uint32_t flash_size;
}iap_info_type;

void iap_init(void);
error_status iap_get_upgrade_flag(void);
void iap_loop(void);
void jump_to_app(uint32_t address);
error_status iap_check_upgrade(void);
error_status iap_upgrade_process(void);
/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif


